//-----------------------------------------------------------------------------
// File:     HelloMath2.cpp
// Class:    None
// Parent:   None
// Children: None
// Purpose:  Retrieves user input angle and computes various mathematical calculates and writes to file
//-----------------------------------------------------------------------------
// The following are standard C/C++ header files.
// If a filename is enclosed inside < >  it means the header file is in the Include directory.
// If a filename is enclosed inside " "  it means the header file is in the current directory.
#include <ctype.h>      // Character Types
#include <math.h>       // Mathematical Constants
#include <stdarg.h>     // Variable Argument Lists
#include <stdio.h>      // Standard Input/Output Functions
#include <stdlib.h>     // Utility Functions
#include <string.h>     // String Operations
#include <signal.h>     // Signals (Contol-C + Unix System Calls)
#include <setjmp.h>     // Nonlocal Goto (For Control-C)
#include <time.h>       // Time and Date information
#include <assert.h>     // Verify Program Assertion
#include <errno.h>      // Error Codes (Used in Unix system())
#include <float.h>      // Floating Point Constants
#include <limits.h>     // Implementation Constants
#include <stddef.h>     // Standard Definitions
#include <exception>    // Exception handling (e.g., try, catch throw)
#include <iostream>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <fstream>
using namespace std;
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Prototypes for local functions (functions not called by code in other files)
//-----------------------------------------------------------------------------
double ConvertFromRadiansToDegrees( long angleInDegrees) { return angleInDegrees * 0.017453292519943295769236907684886127134428718885417254560971914402; }
double ConvertFromDegreesToRadians( long angleInRadians) { return angleInRadians * 57.295779513082320876798154814105170332405472466564321549160243861; }
bool   WriteDoubleToFileWithExponentInSpecifiedPrecision( double x, int precision, FILE *fptr );
bool  GetStringFromFile( char inputString[], unsigned long sizeOfString, FILE* fptr);
bool  GetStringFromKeyboard( char inputString[], unsigned long sizeOfString);
bool  WriteStringToScreen ( const char outputString[]);
bool  WriteStringToFile( const char outputString[], FILE *fptr);
const char* ConvertStringToLong( const char *s, long &returnValue, long defaultValue);
bool  WriteDoubleToFileWithExponentInSpecifiedPrecision( double x, int precision, FILE *fptr );FILE* FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute);
//-----------------------------------------------------------------------------
int  main( int numberOfCommandLineArguments, char *arrayOfCommandLineArguments[] )
{   
   //Prompts user to enter a angle in degrees
   WriteStringToScreen("Please enter an integer between 180 and 720:\n");
	
   //Variable to retrieve input form user
   char inputString[256];
   
   //Returns user's input as a string
   GetStringFromKeyboard(inputString,256); 	
   
   //Converts string input to double and stores value in angleInDegrees variable
   //and if input was not an angle function will returns a message stating a default 
   //value will be used.
   long angleInDegrees;
   if(ConvertStringToLong(inputString,angleInDegrees,360)==NULL)
   {
      WriteStringToScreen("A default value of 360 (for an angle in degrees) will be  used\n");
   }
   if(angleInDegrees <180 || angleInDegrees >720 ) 
   {
      angleInDegrees=360;
		WriteStringToScreen("A default value of 360 (for an angle in degrees) will be  used\n");
   }
   
   //Initializes variable that will store the user's input for precicion of calculation.
   char precision[256];
   
   //Prompmts user for precision
   WriteStringToScreen("Please enter the desired precision as an integer\n");
   
   //Retrieves user's input and stores it in the character array precision
   GetStringFromKeyboard(precision,256);
   
   //Initializes a variable to store the numeric value of precision
   long returnValueForPrecision;

   //Converts character to double
   if(ConvertStringToLong(precision,returnValueForPrecision,5)==NULL)
   {
      WriteStringToScreen("A default value of 5 for precision will be  used\n");
   }
   
   //Ensures that the value entered for precision was between 1 and 17 and if not informs user a default
   //value will be used.
   if(returnValueForPrecision <1 || returnValueForPrecision >17 ) 
   {
      returnValueForPrecision=5;
		WriteStringToScreen("A default value of 5 for precision will be  used\n");
   }
	
   
   //Open file called HelloMath2.txt in order  and returns error message if not possible
   FILE *pFile = FileOpenWithMessageIfCannotOpen("HelloMath2.txt","wb");
   if (!pFile) return 1;
   
   //Writes sine, cosine, tangent of angle in radians to file and screen,
   //along with angle in radians and degrees with specified preicsion.
   
   // Create for-loop starting at 0, ending at angleInDegrees, and incrementing by +1
   for (int i=1;i<=angleInDegrees;i++)
   {
      //Writes sine, cosine, tangent of angle in radians to file and screen,
      //along with angle in radians and degrees with specified preicsion.
      WriteDoubleToFileWithExponentInSpecifiedPrecision(i,returnValueForPrecision,pFile);
		double angleInRadians=ConvertFromDegreesToRadians(i);
      WriteDoubleToFileWithExponentInSpecifiedPrecision(angleInRadians,returnValueForPrecision,pFile);
      WriteDoubleToFileWithExponentInSpecifiedPrecision(sin(angleInRadians),returnValueForPrecision,pFile);
      WriteDoubleToFileWithExponentInSpecifiedPrecision( cos(angleInRadians),returnValueForPrecision,pFile); 
      WriteDoubleToFileWithExponentInSpecifiedPrecision(tan(angleInRadians),returnValueForPrecision,pFile);
		
      //Starts a new line for the next angle (i+1)
      WriteStringToFile("\n",pFile);
   }
	 
   //Next line will close the line once the writing is complete.
   fclose (pFile);
   
   // Keep the screen displayed until the user presses the Enter key
   WriteStringToScreen( "\n\n Press  Enter  to terminate the program: " );
   int key = getchar();
   
   // A normal program exit returns 0 (other return values usually signal an error)
   return 0;
}


///___________________________________________________________________________________________
bool  WriteStringToScreen ( const char outputString[])
{
	//Writes the string input from outputString to file, in this case the screen.
   WriteStringToFile(outputString, stdout);
	return 0;
}


//___________________________________________________________________________________
bool  WriteStringToFile( const char outputString[], FILE *fptr)
{
	//Writes the string input from outputString to file, in this case the screen.
   fprintf(fptr,outputString);
	return 0;
}


//____________________________________________________________________________________________________________
const char* ConvertStringToLong( const char *s, long &returnValue, long defaultValue)
{
   
	// Default return value (in case the string is not a valid number)
   returnValue = defaultValue;
   
   // Check if  s  is a NULL string or  "abc"  or  "123huh"  or  "(&junk#") and if not convert string to double 
   if (s)
   {
      char *pointerToCharacterAfterNumber = NULL;
      long x =strtol(s,&pointerToCharacterAfterNumber,0);
      char characterAfterNumber = pointerToCharacterAfterNumber ? *pointerToCharacterAfterNumber : 'z';
      if ( characterAfterNumber == '\0' || isspace (characterAfterNumber) )
      {
         returnValue = x;
			return pointerToCharacterAfterNumber;
      }
   }
   return NULL;
}


//___________________________________________________________________________________________
bool  WriteDoubleToFileWithExponentInSpecifiedPrecision( double x, int precision, FILE *fptr )
{
   // Ensure the precision (number of digits in the mantissa after the decimal point) makes sense.
   // Next, calculate the field width so it includes one extra space to the right of the number.
   if( precision < 0 || precision > 17 ) precision = 15;
   int fieldWidth = precision + 8;

   // Create the format specifier and print the number
   char format[20];
   sprintf( format, " %%- %d.%dE", fieldWidth, precision );
   
   //Returns a error message if unable to write double
   return fprintf( fptr, format, x) >0;
}


//___________________________________________________________________________________________
FILE* FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute)

//Open files for writing and if not, returns an error message. 
{
	FILE *p2File;
	p2File=fopen(filename,attribute);
	if (p2File==NULL)
	{
		WriteStringToScreen("Sorry, could not open file that was requested, please check name and re-enter\n");
		return NULL;
	}
	return p2File;
}


//__________________________________________________________________________________
bool GetStringFromKeyboard( char inputString[], unsigned long sizeOfString)
{ 
	//Returns the users input from the keyboard to the character inputString
   GetStringFromFile(inputString,256,stdin);
	 return 0;
}   


//_________________________________________________________________________________
bool GetStringFromFile(char inputString[], unsigned long sizeOfString, FILE* fptr)
{
	//Returns the users input from the keyboard and stores it in the inputString
   fgets (inputString, sizeOfString, fptr );
	return 0;
}


//_____________________________________________________________________________________